'use strict';

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

var _konsul = require('../shared/konsul');

var _debounce = require('../shared/debounce');

var _debounce2 = _interopRequireDefault(_debounce);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _objectWithoutProperties(obj, keys) { var target = {}; for (var i in obj) { if (keys.indexOf(i) >= 0) continue; if (!Object.prototype.hasOwnProperty.call(obj, i)) continue; target[i] = obj[i]; } return target; }

var ReactFiberReconciler = require('react-dom/lib/ReactFiberReconciler');


var emptyObject = {};

var KonsulReconciler = ReactFiberReconciler({
  createInstance: function createInstance(type, allProps, rootContainerInstance, hostContext, internalInstanceHandle) {
    var children = allProps.children,
        props = _objectWithoutProperties(allProps, ['children']);

    var instance = (0, _konsul.makeElement)({ type: type }, rootContainerInstance);
    (0, _konsul.applyProps)(instance, { type: type, props: props });

    return instance;
  },
  appendInitialChild: function appendInitialChild(parentInstance, child) {
    parentInstance.append(child);
    child.parent = parentInstance;
    child.setOrder(parentInstance.children.length);
  },
  appendChild: function appendChild(parentInstance, child) {
    parentInstance.append(child);
    child.parent = parentInstance;
    child.setOrder(parentInstance.children.length);
  },
  removeChild: function removeChild(parentInstance, child) {
    child.destroy();
  },
  insertBefore: function insertBefore(parentInstance, child, beforeChild) {
    // TODO: Not working properly yet
    parentInstance.append(child);
    child.parent = parentInstance;
    console.log(beforeChild.order);

    if (beforeChild) {
      child.setOrder(beforeChild.order);
    }
  },
  finalizeInitialChildren: function finalizeInitialChildren(instance, type, allProps, rootContainerInstance) {
    var children = allProps.children,
        props = _objectWithoutProperties(allProps, ['children']);

    (0, _konsul.applyProps)(instance, { type: type, props: props });
    return false;
  },
  prepareUpdate: function prepareUpdate(instance, type, oldProps, newProps, rootContainerInstance, hostContext) {
    return newProps;
  },
  commitUpdate: function commitUpdate(instance, updatePayload, type, oldProps, newProps, internalInstanceHandle) {
    (0, _konsul.applyProps)(instance, { type: type, props: newProps });
    instance.konsul.debounceRender();
  },
  commitMount: function commitMount(instance, type, newProps, internalInstanceHandle) {
    instance.konsul.debounceRender();
  },
  getRootHostContext: function getRootHostContext(rootContainerInstance) {
    return emptyObject;
  },
  getChildHostContext: function getChildHostContext(parentHostContext, type) {
    return emptyObject;
  },
  getPublicInstance: function getPublicInstance(instance) {
    return instance;
  },
  shouldSetTextContent: function shouldSetTextContent(props) {
    return false;
  },
  resetTextContent: function resetTextContent(instance) {
    instance.setContent('');
  },
  createTextInstance: function createTextInstance(text, rootContainerInstance, hostContext, internalInstanceHandle) {
    var instance = rootContainerInstance.text();
    instance.setChildren([text]);
    return instance;
  },
  commitTextUpdate: function commitTextUpdate(textInstance, oldText, newText) {
    textInstance.setChildren([newText]);
  },
  prepareForCommit: function prepareForCommit() {
    // noop
  },
  resetAfterCommit: function resetAfterCommit() {
    // noop
  },
  scheduleAnimationCallback: function scheduleAnimationCallback() {
    throw new Error('Unimplemented');
  },
  scheduleDeferredCallback: function scheduleDeferredCallback(a) {
    throw new Error('Unimplemented');
  },

  useSyncScheduling: true
});

module.exports = function (element, konsul, callback) {
  var root = roots.get(konsul);
  if (!root) {
    root = KonsulReconciler.createContainer(konsul);
    roots.set(konsul, root);
  }

  konsul.removeAllListeners('shouldRender');

  var debouncer = (0, _debounce2.default)(function () {
    return konsul.render();
  }, 16);
  konsul.debounceRender = function () {
    return debouncer();
  };
  KonsulReconciler.updateContainer(element, root, null, callback);
  konsul.debounceRender();

  return KonsulReconciler.getPublicRootInstance(root);
};

var roots = new Map();

function _inspect(input, depth) {
  var maxDepth = 4;
  var maxKeys = 15;

  if (depth === undefined) {
    depth = 0;
  }

  depth += 1;

  if (input === null) {
    return 'null';
  } else if (input === undefined) {
    return 'void';
  } else if (typeof input === 'string' || typeof input === 'number' || typeof input === 'boolean') {
    return typeof input === 'undefined' ? 'undefined' : _typeof(input);
  } else if (Array.isArray(input)) {
    if (input.length > 0) {
      if (depth > maxDepth) return '[...]';

      var first = _inspect(input[0], depth);

      if (input.every(function (item) {
        return _inspect(item, depth) === first;
      })) {
        return first.trim() + '[]';
      } else {
        return '[' + input.slice(0, maxKeys).map(function (item) {
          return _inspect(item, depth);
        }).join(', ') + (input.length >= maxKeys ? ', ...' : '') + ']';
      }
    } else {
      return 'Array';
    }
  } else {
    var keys = Object.keys(input);

    if (!keys.length) {
      if (input.constructor && input.constructor.name && input.constructor.name !== 'Object') {
        return input.constructor.name;
      } else {
        return 'Object';
      }
    }

    if (depth > maxDepth) return '{...}';
    var indent = '  '.repeat(depth - 1);
    var entries = keys.slice(0, maxKeys).map(function (key) {
      return (/^([A-Z_$][A-Z0-9_$]*)$/i.test(key) ? key : JSON.stringify(key)) + ': ' + _inspect(input[key], depth) + ';';
    }).join('\n  ' + indent);

    if (keys.length >= maxKeys) {
      entries += '\n  ' + indent + '...';
    }

    if (input.constructor && input.constructor.name && input.constructor.name !== 'Object') {
      return input.constructor.name + ' {\n  ' + indent + entries + '\n' + indent + '}';
    } else {
      return '{\n  ' + indent + entries + '\n' + indent + '}';
    }
  }
}