'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

exports.makeElement = makeElement;
exports.applyProps = applyProps;
var STATUS_NO_CHANGE = 0;
var STATUS_CREATED = 1;
var STATUS_UPDATED = 2;

function makeElement(node, konsul) {
  if (!(node instanceof Object)) {
    throw new TypeError('Value of argument "node" violates contract.\n\nExpected:\nObject\n\nGot:\n' + _inspect(node));
  }

  switch (node.type) {
    case 'text':
    case 'image':
    case 'container':
    case 'group':
    case 'button':
      return konsul[node.type](konsul);
    default:
      return null;
  }
}

function applyProps(element, node, prevNode) {
  if (!(node instanceof Object)) {
    throw new TypeError('Value of argument "node" violates contract.\n\nExpected:\nObject\n\nGot:\n' + _inspect(node));
  }

  if (!(prevNode === undefined || prevNode == null || prevNode instanceof Object)) {
    throw new TypeError('Value of optional argument "prevNode" violates contract.\n\nExpected:\n?Object\n\nGot:\n' + _inspect(prevNode));
  }

  var type = node.type,
      props = node.props;


  switch (type) {
    case 'text':
      if (shouldPropUpdate('style', node, prevNode)) {
        element.setStyle(props.style);
      }
      if (shouldPropUpdate('type', node, prevNode)) {
        element.setType(props.type);
      }
      break;
    case 'image':
      if (shouldPropUpdate('style', node, prevNode)) {
        element.setStyle(props.style);
      }

      if (shouldPropUpdate('source', node, prevNode)) {
        element.setSource(props.source);
      }
      break;
    case 'group':
      if (shouldPropUpdate('name', node, prevNode)) {
        element.setName(props.name);
      }
      break;
    case 'button':
      if (shouldPropUpdate('label', node, prevNode)) {
        element.setLabel(props.label);
      }

      var clickStatus = shouldPropUpdate('onClick', node, prevNode);
      if (clickStatus) {
        element.removeAllListeners('click');
        element.on('click', props.onClick);
      }
      break;
  }
}

function shouldPropUpdate(prop, nextNode, prevNode) {
  if (!(typeof prop === 'string')) {
    throw new TypeError('Value of argument "prop" violates contract.\n\nExpected:\nstring\n\nGot:\n' + _inspect(prop));
  }

  if (!(nextNode instanceof Object)) {
    throw new TypeError('Value of argument "nextNode" violates contract.\n\nExpected:\nObject\n\nGot:\n' + _inspect(nextNode));
  }

  if (!(prevNode === undefined || prevNode == null || prevNode instanceof Object)) {
    throw new TypeError('Value of optional argument "prevNode" violates contract.\n\nExpected:\n?Object\n\nGot:\n' + _inspect(prevNode));
  }

  if (prevNode && typeof prevNode.props[prop] !== 'undefined') {
    return nextNode.props[prop] !== prevNode.props[prop] ? STATUS_UPDATED : STATUS_NO_CHANGE;
  }

  return typeof nextNode.props[prop] !== 'undefined' ? STATUS_CREATED : STATUS_NO_CHANGE;
}

function _inspect(input, depth) {
  var maxDepth = 4;
  var maxKeys = 15;

  if (depth === undefined) {
    depth = 0;
  }

  depth += 1;

  if (input === null) {
    return 'null';
  } else if (input === undefined) {
    return 'void';
  } else if (typeof input === 'string' || typeof input === 'number' || typeof input === 'boolean') {
    return typeof input === 'undefined' ? 'undefined' : _typeof(input);
  } else if (Array.isArray(input)) {
    if (input.length > 0) {
      if (depth > maxDepth) return '[...]';

      var first = _inspect(input[0], depth);

      if (input.every(function (item) {
        return _inspect(item, depth) === first;
      })) {
        return first.trim() + '[]';
      } else {
        return '[' + input.slice(0, maxKeys).map(function (item) {
          return _inspect(item, depth);
        }).join(', ') + (input.length >= maxKeys ? ', ...' : '') + ']';
      }
    } else {
      return 'Array';
    }
  } else {
    var keys = Object.keys(input);

    if (!keys.length) {
      if (input.constructor && input.constructor.name && input.constructor.name !== 'Object') {
        return input.constructor.name;
      } else {
        return 'Object';
      }
    }

    if (depth > maxDepth) return '{...}';
    var indent = '  '.repeat(depth - 1);
    var entries = keys.slice(0, maxKeys).map(function (key) {
      return (/^([A-Z_$][A-Z0-9_$]*)$/i.test(key) ? key : JSON.stringify(key)) + ': ' + _inspect(input[key], depth) + ';';
    }).join('\n  ' + indent);

    if (keys.length >= maxKeys) {
      entries += '\n  ' + indent + '...';
    }

    if (input.constructor && input.constructor.name && input.constructor.name !== 'Object') {
      return input.constructor.name + ' {\n  ' + indent + entries + '\n' + indent + '}';
    } else {
      return '{\n  ' + indent + entries + '\n' + indent + '}';
    }
  }
}